import 'dart:async';

import 'package:dio/dio.dart';
import '../../configs/application.dart';
import '../../utils/logger.dart';

enum ApiMethod { GET, POST, PUT, DELETE, PATH }

class ApiClient {
  factory ApiClient() {
    return _instance;
  }

  ApiClient._internal();
  static final ApiClient _instance = ApiClient._internal();

  static BaseOptions baseOptions = BaseOptions(
    baseUrl: Application.domain,
    connectTimeout: 10000,
    receiveTimeout: 10000,
    responseType: ResponseType.json,
  );

  static BaseOptions exportOption() {
    baseOptions.headers['content-type'] = 'application/json';
    baseOptions.baseUrl = Application.domain;
    baseOptions.headers['api-key'] = Application.key;
    return baseOptions;
  }

  static Future<Response> connect(
    ApiMethod method,
    String url, {
    Map<String, String>? headers,
    Map<String, String>? query,
    Map<String, dynamic>? body,
    FormData? data,
  }) async {
    try {
      Response response;
      final dio = Dio(exportOption());
      UtilLogger.log('url $url, query $query method $method');
      switch (method) {
        case ApiMethod.GET:
          response = await dio.get(url, queryParameters: query);
          break;
        case ApiMethod.POST:
          response = await dio.post(
            url,
            queryParameters: query,
            data: data ?? body,
          );
          break;
        case ApiMethod.DELETE:
          response = await dio.delete(url, queryParameters: query);
          break;
        default:
          response = await dio.get(url, queryParameters: query);
          break;
      }
      return response;
    } on DioError catch (error) {
      UtilLogger.log(error);
      throw createErrorEntity(error);
    }
  }
}

ErrorEntity createErrorEntity(DioError error) {
  switch (error.type) {
    case DioErrorType.cancel:
      return ErrorEntity(code: -1, message: 'Request cancellation');

    case DioErrorType.connectTimeout:
      return ErrorEntity(code: -1, message: 'Connection timed out');

    case DioErrorType.sendTimeout:
      return ErrorEntity(code: -1, message: 'Request timed out');

    case DioErrorType.receiveTimeout:
      return ErrorEntity(code: -1, message: 'Response timeout');

    case DioErrorType.response:
      try {
        var errCode = error.response?.statusCode;
        if (errCode == null) {
          return ErrorEntity(code: -2, message: error.message);
        }
        switch (errCode) {
          case 400:
            return ErrorEntity(code: errCode, message: 'Bad Request');

          case 401:
            return ErrorEntity(code: errCode, message: 'Unauthorized');

          case 403:
            return ErrorEntity(code: errCode, message: 'Forbidden');

          case 404:
            return ErrorEntity(code: errCode, message: 'Not Found');

          case 405:
            return ErrorEntity(code: errCode, message: 'Method Not Allowed');

          case 500:
            return ErrorEntity(code: errCode, message: 'Internal Server Error');

          case 502:
            return ErrorEntity(code: errCode, message: 'Bad Gateway');

          case 503:
            return ErrorEntity(code: errCode, message: 'Service Unavailable');

          case 505:
            return ErrorEntity(code: errCode, message: 'HTTP  Not Supported');

          default:
            return ErrorEntity(
              code: errCode,
              message: error.response?.statusMessage ?? '',
            );
        }
      } on Exception catch (error) {
        return ErrorEntity(code: -1, message: error.toString());
      }

    default:
      return ErrorEntity(code: -1, message: error.message);
  }
}

class ErrorEntity implements Exception {
  int code;
  String? message;
  ErrorEntity({required this.code, this.message});

  @override
  String toString() {
    if (message == null) return 'Exception';
    return 'Exception: code $code, $message';
  }
}
