import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:camera/camera.dart';
import 'package:path/path.dart' as path;
import 'package:pedantic/pedantic.dart';

import '../constants/constants.dart';

import 'camera_picker.dart';
import 'package:get/get.dart';

typedef EntitySaveCallback = FutureOr<dynamic> Function({
  BuildContext context,
  File file,
});

class CameraPickerViewer extends StatefulWidget {
  const CameraPickerViewer({
    Key? key,
    required this.pickerState,
    required this.previewXFile,
    required this.theme,
    this.shouldDeletePreviewFile = false,
    this.onEntitySaving,
  }) : super(key: key);

  /// State of the picker.
  final CameraPickerState pickerState;

  /// The [XFile] of the preview file.
  final XFile previewXFile;

  /// The [ThemeData] which the picker is using.
  final ThemeData theme;

  /// Whether the preview file will be delete when pop.
  final bool shouldDeletePreviewFile;

  /// {@macro wechat_camera_picker.SaveEntityCallback}
  final EntitySaveCallback? onEntitySaving;

  /// Static method to push with the navigator.
  static Future<AssetEntity?> pushToViewer(
    BuildContext context, {
    required CameraPickerState pickerState,
    required XFile previewXFile,
    required ThemeData theme,
    bool shouldDeletePreviewFile = false,
    EntitySaveCallback? onEntitySaving,
  }) {
    return Navigator.of(context).push<AssetEntity?>(
      PageRouteBuilder<AssetEntity?>(
        pageBuilder: (_, __, ___) => CameraPickerViewer(
          pickerState: pickerState,
          previewXFile: previewXFile,
          theme: theme,
          shouldDeletePreviewFile: shouldDeletePreviewFile,
          onEntitySaving: onEntitySaving,
        ),
        transitionsBuilder: (
          BuildContext context,
          Animation<double> animation,
          Animation<double> secondaryAnimation,
          Widget child,
        ) {
          return FadeTransition(opacity: animation, child: child);
        },
      ),
    );
  }

  @override
  _CameraPickerViewerState createState() => _CameraPickerViewerState();
}

class _CameraPickerViewerState extends State<CameraPickerViewer> {
  CameraPickerState get pickerState => widget.pickerState;

  XFile get previewXFile => widget.previewXFile;

  /// Construct an [File] instance through [previewXFile].
  File get previewFile => File(previewXFile.path);

  ThemeData get theme => widget.theme;

  bool get shouldDeletePreviewFile => widget.shouldDeletePreviewFile;

  Future<void> createAssetEntityAndPop() async {
    if (widget.onEntitySaving != null) {
      await widget.onEntitySaving!(
        context: context,
        file: previewFile,
      );
      return;
    }
    Future<AssetEntity?> saveFuture;

    final data = await previewFile.readAsBytes();
    saveFuture = PhotoManager.editor.saveImage(
      data,
      title: path.basename(previewFile.path),
    );

    unawaited(saveFuture.then((AssetEntity? entity) {
      if (shouldDeletePreviewFile && previewFile.existsSync()) {
        previewFile.delete();
      }
      Navigator.of(context).pop(entity);
    }));
  }

  /// The back button for the preview section.
  Widget get previewBackButton {
    return InkWell(
      borderRadius: maxBorderRadius,
      onTap: () {
        if (previewFile.existsSync()) {
          previewFile.delete();
        }
        Navigator.of(context).pop();
      },
      child: Container(
        margin: const EdgeInsets.all(10.0),
        width: Screens.width / 15,
        height: Screens.width / 15,
        decoration: const BoxDecoration(
          color: Colors.white,
          shape: BoxShape.circle,
        ),
        child: const Center(
          child: Icon(
            Icons.keyboard_return_rounded,
            color: Colors.black,
            size: 18.0,
          ),
        ),
      ),
    );
  }

  /// The confirm button for the preview section.
  Widget get previewConfirmButton {
    return MaterialButton(
      minWidth: 20.0,
      height: 32.0,
      padding: const EdgeInsets.symmetric(horizontal: 20.0),
      color: Colors.blue,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(3.0),
      ),
      onPressed: createAssetEntityAndPop,
      materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
      child: Text(
        'Confirm'.tr,
        style: TextStyle(
          color: theme.textTheme.bodyText1?.color,
          fontSize: 17.0,
          fontWeight: FontWeight.normal,
        ),
      ),
    );
  }

  Widget get viewerActions {
    return SafeArea(
      child: Padding(
        padding: const EdgeInsets.symmetric(
          horizontal: 8.0,
          vertical: 20.0,
        ),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Row(
              children: <Widget>[
                previewBackButton,
                const Spacer(),
              ],
            ),
            Row(
              children: <Widget>[
                const Spacer(),
                previewConfirmButton,
              ],
            ),
          ],
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.black,
      child: Stack(
        children: <Widget>[
          Positioned.fill(child: Image.file(previewFile)),
          viewerActions,
        ],
      ),
    );
  }
}
